/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.share

import android.content.Context
import com.kakao.sdk.network.ApiCallback
import com.kakao.sdk.network.ApiFactory
import com.kakao.sdk.share.ShareClient
import com.kakao.sdk.share.model.SharingResult
import com.kakao.sdk.share.model.ValidationResult
import com.kakao.sdk.template.model.DefaultTemplate

/**
 * @suppress
 */
private val ShareClient.partnerApi: PartnerShareApi
    get() = ApiFactory.kapi.create(PartnerShareApi::class.java)

/**
 * 카카오 디벨로퍼스에서 생성한 메시지 템플릿을 카카오톡으로 공유.
 * 템플릿을 생성하는 방법은 [메시지 템플릿 가이드](https://developers.kakao.com/docs/latest/ko/message/message-template) 참고.
 *
 * @param targetAppKey 말풍선 출처 영역에 보여질 서비스 앱의 네이티브 키.
 *
 * @see [ShareClient.shareCustom]
 */
@JvmOverloads
fun ShareClient.shareCustomForPartner(
    context: Context,
    targetAppKey: String,
    templateId: Long,
    templateArgs: Map<String, String>? = null,
    serverCallbackArgs: Map<String, String>? = null,
    callback: (sharingResult: SharingResult?, error: Throwable?) -> Unit
) {
    partnerApi.validateCustom(targetAppKey, templateId, templateArgs)
        .enqueue(object : ApiCallback<ValidationResult>() {
            override fun onComplete(model: ValidationResult?, error: Throwable?) {
                if (model != null) {
                    try {
                        callback(
                            kakaotalkShareIntentClient.sharingResultFromResponse(
                                context,
                                model,
                                serverCallbackArgs,
                                targetAppKey
                            ),
                            null
                        )
                    } catch (e: Throwable) {
                        callback(null, e)
                    }
                } else {
                    callback(null, error)
                }
            }
        })
}

/**
 * 기본 템플릿을 카카오톡으로 공유.
 *
 * @param targetAppKey 말풍선 출처 영역에 보여질 서비스 앱의 네이티브 키.
 *
 * @see [ShareClient.shareDefault]
 */
@JvmOverloads
fun ShareClient.shareDefaultForPartner(
    context: Context,
    targetAppKey: String,
    defaultTemplate: DefaultTemplate,
    serverCallbackArgs: Map<String, String>? = null,
    callback: (sharingResult: SharingResult?, error: Throwable?) -> Unit
) {
    partnerApi.validateDefault(targetAppKey, defaultTemplate)
        .enqueue(object : ApiCallback<ValidationResult>() {
            override fun onComplete(model: ValidationResult?, error: Throwable?) {
                if (model != null) {
                    try {
                        callback(
                            kakaotalkShareIntentClient.sharingResultFromResponse(
                                context,
                                model,
                                serverCallbackArgs,
                                targetAppKey
                            ),
                            null
                        )
                    } catch (e: Throwable) {
                        callback(null, e)
                    }
                } else {
                    callback(null, error)
                }
            }
        })
}

/**
 * 지정된 URL 을 스크랩하여 만들어진 템플릿을 카카오톡으로 공유.
 *
 * @param targetAppKey 말풍선 출처 영역에 보여질 서비스 앱의 네이티브 키.
 *
 * @see [ShareClient.shareScrap]
 */
@JvmOverloads
fun ShareClient.shareScrapForPartner(
    context: Context,
    targetAppKey: String,
    url: String,
    templateId: Long? = null,
    templateArgs: Map<String, String>? = null,
    serverCallbackArgs: Map<String, String>? = null,
    callback: (sharingResult: SharingResult?, error: Throwable?) -> Unit
) {
    partnerApi.validateScrap(targetAppKey, url, templateId, templateArgs)
        .enqueue(object : ApiCallback<ValidationResult>() {
            override fun onComplete(model: ValidationResult?, error: Throwable?) {
                if (model != null) {
                    try {
                        callback(
                            kakaotalkShareIntentClient.sharingResultFromResponse(
                                context,
                                model,
                                serverCallbackArgs,
                                targetAppKey
                            ),
                            null
                        )
                    } catch (e: Throwable) {
                        callback(null, e)
                    }
                } else {
                    callback(null, error)
                }
            }
        })
}
