/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.talk.model

import android.net.Uri
import android.os.Parcelable
import com.google.gson.annotations.SerializedName
import com.kakao.sdk.common.json.UnknownValue
import com.kakao.sdk.talk.Constants
import com.kakao.sdk.talk.model.FriendOrder
import com.kakao.sdk.talk.model.Order
import kotlinx.parcelize.Parcelize

/**
 * 파트너용으로 제공되는 친구 정보입니다.
 *
 * @property id 회원번호. [appRegistered]이 false 이면 null 이 됩니다.
 * @property uuid 메시지를 전송하기 위한 고유 아이디
 * @property profileNickname 친구의 대표 프로필 닉네임. 앱 가입친구의 경우 앱에서 설정한 닉네임. 미가입친구의 경우 톡 또는 스토리의 닉네임
 * @property profileThumbnailImage 친구의 썸네일 이미지 URL
 * @property favorite 즐겨찾기 추가 여부
 * @property serviceUserId 친구의 레거시(Legacy) 회원번호(Service user ID)
 * @property appRegistered 친구의 앱 연결 여부
 * @property relation 카카오톡 및 카카오스토리에서 사용자와 친구의 관계
 * @property talkOs 톡에 가입된 기기의 os 정보 (android / ios)
 * @property allowedMsg 메시지 수신이 허용되었는지 여부. 앱가입 친구의 경우는 feed msg 에 해당. 앱미가입친구는 invite msg 에 해당
 */
@Parcelize
data class PartnerFriend(
    val id: Long?,
    val uuid: String,
    val profileNickname: String?,
    val profileThumbnailImage: String?,
    val favorite: Boolean?,
    val serviceUserId: Long?,

    @Deprecated("해당 property는 'appRegistered'로 대체되었습니다. ")
    val registeredToApp: Boolean?,

    val appRegistered: Boolean?,

    val relation: FriendRelation?,
    val talkOs: String?,

    @Deprecated("해당 property는 'allowedMsg'로 대체되었습니다. ")
    val msgAllowed: Boolean?,

    val allowedMsg: Boolean?
) : Parcelable



/**
 * 친구 목록 조회 Context
 */
@Parcelize
data class PartnerFriendsContext(
        var offset : Int? = null,
        var limit : Int? = null,
        var order : Order? = null,
        var friendType: FriendType? = null,
        var friendOrder : FriendOrder? = null,
        var friendFilter : FriendFilter? = null,
        var countryCodes : List<String>? = null,
        val url: String = ""
) : Parcelable {

    @Throws(IllegalArgumentException::class)
    constructor(url: String) : this(offset = null, limit = null, order = null, friendOrder = null, friendFilter = null, countryCodes = null){

        val uri = Uri.parse(url) ?: throw IllegalArgumentException()

        offset = uri.getQueryParameter(Constants.OFFSET)?.toInt()
        limit = uri.getQueryParameter(Constants.LIMIT)?.toInt()
        order = uri.getQueryParameter(Constants.ORDER)?.let {
            try {
                Order.values().find { order ->
                    order.javaClass.getField(order.name).getAnnotation(SerializedName::class.java)?.value == it
                }
            }
            catch (e: IllegalArgumentException) {
                null
            }
        }

        friendType = uri.getQueryParameter(Constants.FRIEND_TYPE)?.let {
            try {
                FriendType.values().find { friendType ->
                    friendType.javaClass.getField(friendType.name).getAnnotation(SerializedName::class.java)?.value == it
                }
            }
            catch (e: IllegalArgumentException) {
                null
            }
        }

        friendOrder = uri.getQueryParameter(Constants.FRIEND_ORDER)?.let {
            try {
                FriendOrder.values().find { friendOrder ->
                    friendOrder.javaClass.getField(friendOrder.name).getAnnotation(SerializedName::class.java)?.value == it
                }
            }
            catch (e: IllegalArgumentException) {
                null
            }
        }

        friendFilter = uri.getQueryParameter(Constants.FRIEND_FILTER)?.let {
            try {
                FriendFilter.values().find { friendFilter ->
                    friendFilter.javaClass.getField(friendFilter.name).getAnnotation(SerializedName::class.java)?.value == it
                }
            }
            catch (e: IllegalArgumentException) {
                null
            }
        }

        countryCodes = uri.getQueryParameter(Constants.COUNTRY_CODES)?.split(",")
    }
}



/**
 * 친구 목록 필터링
 */
enum class FriendFilter {
    @SerializedName("none")
    NONE,
    @SerializedName("registered")
    REGISTERED,
    @SerializedName("invitable")
    INVITABLE;
}

/**
 * 친구를 가져올 서비스 선택
 */
enum class FriendType {
    /**
     * 카카오톡 친구
     */
    @SerializedName("talk")
    KAKAO_TALK,
    /**
     * 카카오스토리 친구
     */
    @SerializedName("story")
    KAKAO_STORY,
    /**
     * 카카오톡과 카카오스토리 친구
     */
    @SerializedName("talkstory")
    KAKAO_TALK_AND_STORY,
    @UnknownValue
    UNKNOWN;
}

@Parcelize
data class FriendRelation(val talk: Relation?, val story: Relation?) : Parcelable

enum class Relation {
    @SerializedName("N/A")
    NONE,
    @SerializedName("friend")
    FRIEND,
    @SerializedName("no_friend")
    NOT_FRIEND,
    @UnknownValue
    UNKNOWN;
}